#version 330

const int MAX_POINT_LIGHTS = 4;

in vec2 texCoord0;
in vec3 normal0;
in vec3 worldPosition0;

struct BaseLight
{
	vec3 color;
	float intensity;
};

struct DirectionalLight
{
	BaseLight base;
	vec3 direction;
};

struct Attenuation
{
	float constant;
	float linear;
	float exponent;
};

struct PointLight
{
	BaseLight base;
	Attenuation atten;
	vec3 position;
};

uniform vec3 baseColor;
uniform vec3 ambientLight;
uniform vec3 cameraPosition;
uniform float specularIntensity;
uniform float specularExponent;
uniform sampler2D sampler;
uniform DirectionalLight directionalLight;
uniform PointLight pointLights[MAX_POINT_LIGHTS];

vec4 calcLight(BaseLight light, vec3 direction, vec3 normal)
{
	float diffuseFactor = dot(normal, -direction);
	vec4 diffuseColor = vec4(0, 0, 0, 0);
	vec4 specularColor = vec4(0, 0, 0, 0);

	if(diffuseFactor > 0)
	{
		diffuseColor = vec4(light.color, 1.0) * light.intensity * diffuseFactor;

		vec3 directionToCamera = normalize(cameraPosition - worldPosition0);
		vec3 reflectDirection = normalize(reflect(direction, normal));

		float specularFactor = dot(directionToCamera, reflectDirection);
		specularFactor = pow(specularFactor, specularExponent);

		if(specularFactor > 0)
		{
			specularColor = vec4(light.color, 1.0) * specularIntensity * specularFactor;
		}
	}
	
	return diffuseColor + specularColor;
}

vec4 calcDirectionalLight(DirectionalLight light, vec3 normal)
{
	return calcLight(light.base, -light.direction, normal);
}

vec4 calcPointLight(PointLight light, vec3 normal)
{
	vec3 direction = worldPosition0 - light.position;
	float distanceToPoint = length(direction);
	direction = normalize(direction);

	vec4 color = calcLight(light.base, direction, normal);

	float attenuation = light.atten.constant + light.atten.linear * distanceToPoint + light.atten.exponent * distanceToPoint * distanceToPoint + 0.0001;

	return color / attenuation;
}

void main()
{
	vec4 textureColor = texture2D(sampler, texCoord0.xy);
	vec4 color = vec4(baseColor, 1);
	vec4 totalLight = vec4(ambientLight, 1);
	
	if(textureColor != vec4(0, 0, 0, 0))
		color *= textureColor;
	
	vec3 normal = normalize(normal0);

	totalLight += calcDirectionalLight(directionalLight, normal);
	
	for(int i = 0; i < MAX_POINT_LIGHTS; i++)
	{
		totalLight += calcPointLight(pointLights[i], normal);
	}
	
	gl_FragColor = color * totalLight;
}